// Copyright 2019 Amazon.com, Inc. or its affiliates. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#include "rosbag2_storage/storage_interfaces/base_io_interface.hpp"

namespace rosbag2_storage
{
namespace storage_interfaces
{
const uint64_t MAX_BAGFILE_SIZE_NO_SPLIT = 0;
const uint64_t MAX_BAGFILE_DURATION_NO_SPLIT = 0;
}
}
